@@ -42,6 +42,36 @@ if ( ! class_exists( 'um\admin\core\Admin_Users' ) ) {
 		}
 
 
+		function get_users() {
+			UM()->admin()->check_ajax_nonce();
+
+			$search_request = ! empty( $_REQUEST['search'] ) ? sanitize_text_field( $_REQUEST['search'] ) : '';
+			$page           = ! empty( $_REQUEST['page'] ) ? absint( $_REQUEST['page'] ) : 1;
+			$per_page       = 20;
+
+			$args = array(
+				'fields' => array( 'ID', 'user_login' ),
+				'paged'  => $page,
+				'number' => $per_page
+			);
+
+			if ( ! empty( $search_request ) ) {
+				$args['search'] = $search_request;
+			}
+
+			$users_query = new \WP_User_Query( $args );
+			$users       = $users_query->get_results();
+			$total_count = $users_query->get_total();
+
+			wp_send_json_success(
+				array(
+					'users'       => $users,
+					'total_count' => $total_count,
+				)
+			);
+		}
+
+
 		/**
 		 * Restrict the edit/delete users via wp-admin screen by the UM role capabilities
 		 *
@@ -138,7 +168,7 @@ if ( ! class_exists( 'um\admin\core\Admin_Users' ) ) {
 
 				case 'um_delete':
 					if ( is_admin() ) {
-						wp_die( 'This action is not allowed in backend.', 'ultimate-member' );
+						wp_die( __( 'This action is not allowed in backend.', 'ultimate-member' ) );
 					}
 					UM()->user()->delete();
 					break;
@@ -164,8 +194,8 @@ if ( ! class_exists( 'um\admin\core\Admin_Users' ) ) {
 
 			</div>
 
-			<?php if ( ! empty( $_REQUEST['status'] ) ) { ?>
-				<input type="hidden" name="status" id="um_status" value="<?php echo esc_attr( sanitize_key( $_REQUEST['status'] ) );?>"/>
+			<?php if ( ! empty( $_REQUEST['um_status'] ) ) { ?>
+				<input type="hidden" name="um_status" id="um_status" value="<?php echo esc_attr( sanitize_key( $_REQUEST['um_status'] ) );?>"/>
 			<?php }
 		}
 
@@ -306,13 +336,13 @@ if ( ! class_exists( 'um\admin\core\Admin_Users' ) ) {
 		 * @param $query
 		 * @return mixed
 		 */
-		function sort_by_newest( $query ) {
+		public function sort_by_newest( $query ) {
 			global $pagenow;
 
-			if ( is_admin() && $pagenow == 'users.php' ) {
+			if ( is_admin() && 'users.php' === $pagenow ) {
 				if ( ! isset( $_REQUEST['orderby'] ) ) {
-					$query->query_vars["order"] = 'desc';
-					$query->query_orderby = " ORDER BY user_registered " . ( $query->query_vars["order"] == 'desc' ? 'desc ' : 'asc ' ); //set sort order
+					$query->query_vars['order'] = 'desc';
+					$query->query_orderby       = ' ORDER BY user_registered ' . ( 'desc' === $query->query_vars['order'] ? 'desc ' : 'asc ' ); //set sort order
 				}
 			}
 
@@ -326,14 +356,13 @@ if ( ! class_exists( 'um\admin\core\Admin_Users' ) ) {
 		 * @param $query
 		 * @return mixed
 		 */
-		function filter_users_by_status( $query ) {
+		public function filter_users_by_status( $query ) {
 			global $wpdb, $pagenow;
+			if ( is_admin() && 'users.php' === $pagenow && ! empty( $_REQUEST['um_status'] ) ) {
 
-			if ( is_admin() && $pagenow == 'users.php' && ! empty( $_GET['status'] ) ) {
+				$status = sanitize_key( $_REQUEST['um_status'] );
 
-				$status = sanitize_key( $_GET['status'] );
-
-				if ( $status == 'needs-verification' ) {
+				if ( 'needs-verification' === $status ) {
 					$query->query_where = str_replace('WHERE 1=1',
 						"WHERE 1=1 AND {$wpdb->users}.ID IN (
                                  SELECT {$wpdb->usermeta}.user_id FROM $wpdb->usermeta
@@ -350,7 +379,6 @@ if ( ! class_exists( 'um\admin\core\Admin_Users' ) ) {
 						$query->query_where
 					);
 				}
-
 			}
 
 			return $query;
@@ -361,38 +389,38 @@ if ( ! class_exists( 'um\admin\core\Admin_Users' ) ) {
 		 * Add status links to WP Users List Table
 		 *
 		 * @param $views
-		 * @return array|mixed|void
+		 * @return array
 		 */
-		function add_status_links( $views ) {
+		public function add_status_links( $views ) {
 			remove_filter( 'pre_user_query', array( &$this, 'filter_users_by_status' ) );
 
 			$old_views = $views;
 			$views     = array();
 
-			if ( ! isset( $_REQUEST['role'] ) && ! isset( $_REQUEST['status'] ) ) {
+			if ( ! isset( $_REQUEST['role'] ) && ! isset( $_REQUEST['um_status'] ) ) {
 				$views['all'] = '<a href="' . admin_url( 'users.php' ) . '" class="current">' . __( 'All', 'ultimate-member' ) . ' <span class="count">(' . UM()->query()->count_users() . ')</span></a>';
 			} else {
 				$views['all'] = '<a href="' . admin_url( 'users.php' ) . '">' . __( 'All', 'ultimate-member' ) . ' <span class="count">(' . UM()->query()->count_users() . ')</span></a>';
 			}
 
 			$status = array(
-				'approved'                      => __( 'Approved', 'ultimate-member' ),
-				'awaiting_admin_review'         => __( 'Pending review', 'ultimate-member' ),
-				'awaiting_email_confirmation'   => __( 'Waiting e-mail confirmation', 'ultimate-member' ),
-				'inactive'                      => __( 'Inactive', 'ultimate-member' ),
-				'rejected'                      => __( 'Rejected', 'ultimate-member' )
+				'approved'                    => __( 'Approved', 'ultimate-member' ),
+				'awaiting_admin_review'       => __( 'Pending review', 'ultimate-member' ),
+				'awaiting_email_confirmation' => __( 'Waiting e-mail confirmation', 'ultimate-member' ),
+				'inactive'                    => __( 'Inactive', 'ultimate-member' ),
+				'rejected'                    => __( 'Rejected', 'ultimate-member' ),
 			);
 
 			UM()->query()->count_users_by_status( 'unassigned' );
 
 			foreach ( $status as $k => $v ) {
-				if ( isset( $_REQUEST['status'] ) && sanitize_key( $_REQUEST['status'] ) == $k ) {
+				if ( isset( $_REQUEST['um_status'] ) && sanitize_key( $_REQUEST['um_status'] ) === $k ) {
 					$current = 'class="current"';
 				} else {
 					$current = '';
 				}
 
-				$views[ $k ] = '<a href="' . esc_url( admin_url( 'users.php' ) . '?status=' . $k ) . '" ' . $current . '>' . $v . ' <span class="count">(' . UM()->query()->count_users_by_status( $k ) . ')</span></a>';
+				$views[ $k ] = '<a href="' . esc_url( admin_url( 'users.php' ) . '?um_status=' . $k ) . '" ' . $current . '>' . $v . ' <span class="count">(' . UM()->query()->count_users_by_status( $k ) . ')</span></a>';
 			}
 
 			/**
@@ -430,11 +458,11 @@ if ( ! class_exists( 'um\admin\core\Admin_Users' ) ) {
 
 			// hide filters with not accessible roles
 			if ( ! current_user_can( 'administrator' ) ) {
-				$wp_roles = wp_roles();
+				$wp_roles       = wp_roles();
 				$can_view_roles = um_user( 'can_view_roles' );
 				if ( ! empty( $can_view_roles ) ) {
 					foreach ( $wp_roles->get_names() as $this_role => $name ) {
-						if ( ! in_array( $this_role, $can_view_roles ) ) {
+						if ( ! in_array( $this_role, $can_view_roles, true ) ) {
 							unset( $views[ $this_role ] );
 						}
 					}
@@ -448,20 +476,20 @@ if ( ! class_exists( 'um\admin\core\Admin_Users' ) ) {
 		/**
 		 * Bulk user editing actions
 		 */
-		function um_bulk_users_edit() {
+		public function um_bulk_users_edit() {
 			// bulk edit users
 			if ( ! empty( $_REQUEST['users'] ) && ! empty( $_REQUEST['um_bulkedit'] ) && ! empty( $_REQUEST['um_bulk_action'] ) ) {
 
 				$rolename = UM()->roles()->get_priority_user_role( get_current_user_id() );
-				$role = get_role( $rolename );
+				$role     = get_role( $rolename );
 
-				if ( ! current_user_can( 'edit_users' ) && ! $role->has_cap( 'edit_users' )  ) {
-					wp_die( __( 'You do not have enough permissions to do that.', 'ultimate-member' ) );
+				if ( ! current_user_can( 'edit_users' ) && ! $role->has_cap( 'edit_users' ) ) {
+					wp_die( esc_html__( 'You do not have enough permissions to do that.', 'ultimate-member' ) );
 				}
 
 				check_admin_referer( 'bulk-users' );
 
-				$users = array_map( 'absint', (array) $_REQUEST['users'] );
+				$users       = array_map( 'absint', (array) $_REQUEST['users'] );
 				$bulk_action = current( array_filter( $_REQUEST['um_bulk_action'] ) );
 
 				foreach ( $users as $user_id ) {
@@ -544,8 +572,8 @@ if ( ! class_exists( 'um\admin\core\Admin_Users' ) ) {
 				$uri = add_query_arg( 's', sanitize_text_field( $_REQUEST['s'] ), $uri );
 			}
 
-			if ( ! empty( $_REQUEST['status'] ) ) {
-				$uri = add_query_arg( 'status', sanitize_key( $_REQUEST['status'] ), $uri );
+			if ( ! empty( $_REQUEST['um_status'] ) ) {
+				$uri = add_query_arg( 'um_status', sanitize_key( $_REQUEST['um_status'] ), $uri );
 			}
 
 			return $uri;
@@ -553,4 +581,4 @@ if ( ! class_exists( 'um\admin\core\Admin_Users' ) ) {
 		}
 
 	}
-}
\ No newline at end of file
+}
