@@ -199,41 +199,50 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 		 *
 		 * @return array
 		 */
-		function before_save_data( $value, $key, $post_id ) {
+		public function before_save_data( $value, $key, $post_id ) {
 
 			$post = get_post( $post_id );
 
-			if ( $post->post_type == 'um_directory' ) {
+			if ( 'um_directory' === $post->post_type ) {
 
-				if ( ! empty( $value ) && in_array( $key, array( '_um_view_types', '_um_roles', '_um_roles_can_search', '_um_roles_can_filter' ) ) ) {
+				if ( ! empty( $value ) && in_array( $key, array( '_um_view_types', '_um_roles', '_um_roles_can_search', '_um_roles_can_filter' ), true ) ) {
 					$value = array_keys( $value );
-				} elseif ( $key == '_um_search_filters' ) {
+				} elseif ( '_um_search_filters' === $key ) {
 
 					$temp_value = array();
 
 					if ( ! empty( $value ) ) {
 						foreach ( $value as $k ) {
 							$filter_type = $this->filter_types[ $k ];
-							if ( ! empty( $filter_type  ) ) {
-								if ( $filter_type == 'slider' ) {
+							if ( ! empty( $filter_type ) ) {
+								if ( 'slider' === $filter_type ) {
 									if ( ! empty( $_POST[ $k ] ) ) {
-										$temp_value[ $k ] = $_POST[ $k ];
+										if ( count( $_POST[ $k ] ) > 1 ) {
+											$temp_value[ $k ] = array_map( 'intval', $_POST[ $k ] );
+										} else {
+											$temp_value[ $k ] = (int) $_POST[ $k ];
+										}
 									}
-								} elseif ( $filter_type == 'timepicker' || $filter_type == 'datepicker' ) {
+								} elseif ( 'timepicker' === $filter_type || 'datepicker' === $filter_type ) {
 									if ( ! empty( $_POST[ $k . '_from' ] ) && ! empty( $_POST[ $k . '_to' ] ) ) {
-										$temp_value[ $k ] = array( $_POST[ $k . '_from' ], $_POST[ $k . '_to' ] );
+										$temp_value[ $k ] = array(
+											sanitize_text_field( $_POST[ $k . '_from' ] ),
+											sanitize_text_field( $_POST[ $k . '_to' ] ),
+										);
 									}
-								} elseif ( $filter_type == 'select' ) {
+								} elseif ( 'select' === $filter_type ) {
 									if ( ! empty( $_POST[ $k ] ) ) {
 										if ( is_array( $_POST[ $k ] ) ) {
 											$temp_value[ $k ] = array_map( 'trim', $_POST[ $k ] );
 										} else {
 											$temp_value[ $k ] = array( trim( $_POST[ $k ] ) );
 										}
+
+										$temp_value[ $k ] = array_map( 'sanitize_text_field', $temp_value[ $k ] );
 									}
 								} else {
 									if ( ! empty( $_POST[ $k ] ) ) {
-										$temp_value[ $k ] = trim( $_POST[ $k ] );
+										$temp_value[ $k ] = trim( sanitize_text_field( $_POST[ $k ] ) );
 									}
 								}
 							}
@@ -241,7 +250,7 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 					}
 
 					$value = $temp_value;
-				} elseif ( $key == '_um_sorting_fields' ) {
+				} elseif ( '_um_sorting_fields' === $key ) {
 					if ( ! empty( $value['other_data'] ) ) {
 						$other_data = $value['other_data'];
 						unset( $value['other_data'] );
@@ -254,15 +263,15 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 										$metalabel = wp_strip_all_tags( $other_data[ $k ]['label'] );
 									}
 									$row = array(
-										$metakey => ! empty( $metalabel ) ? $metalabel : $metakey
+										$metakey => ! empty( $metalabel ) ? $metalabel : $metakey,
 									);
 								}
 							}
 						}
 					}
-				} elseif ( $key == '_um_sortby_custom' ) {
+				} elseif ( '_um_sortby_custom' === $key ) {
 					$value = sanitize_text_field( $value );
-				} elseif ( $key == '_um_sortby_custom_label' ) {
+				} elseif ( '_um_sortby_custom_label' === $key ) {
 					$value = wp_strip_all_tags( $value );
 				}
 			}
@@ -518,7 +527,7 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 					break;
 				}
 				case 'text': {
-					$filter_from_url = ! empty( $_GET[ 'filter_' . $filter . '_' . $unique_hash ] ) ? $_GET[ 'filter_' . $filter . '_' . $unique_hash ] : $default_value; ?>
+					$filter_from_url = ! empty( $_GET[ 'filter_' . $filter . '_' . $unique_hash ] ) ? sanitize_text_field( $_GET[ 'filter_' . $filter . '_' . $unique_hash ] ) : $default_value; ?>
 						<input type="text" autocomplete="off" id="<?php echo $filter; ?>" name="<?php echo $filter; ?>"
 						   placeholder="<?php esc_attr_e( stripslashes( $attrs['label'] ), 'ultimate-member' ); ?>"
 						          value="<?php echo esc_attr( $filter_from_url ) ?>" class="um-form-field"
@@ -652,11 +661,13 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 
 					ksort( $attrs['options'] );
 
-					$attrs['options'] = apply_filters( 'um_member_directory_filter_select_options_sorted', $attrs['options'], $attrs ); ?>
+					$attrs['options'] = apply_filters( 'um_member_directory_filter_select_options_sorted', $attrs['options'], $attrs );
+
+					$label = isset( $attrs['label'] ) ? $attrs['label'] : ''; ?>
 
 					<select class="um-s1" id="<?php echo esc_attr( $filter ); ?>" name="<?php echo esc_attr( $filter ); ?><?php if ( $admin && count( $attrs['options'] ) > 1 ) { ?>[]<?php } ?>"
-							data-placeholder="<?php esc_attr_e( stripslashes( $attrs['label'] ), 'ultimate-member' ); ?>"
-							aria-label="<?php esc_attr_e( stripslashes( $attrs['label'] ), 'ultimate-member' ); ?>"
+							data-placeholder="<?php esc_attr_e( stripslashes( $label ), 'ultimate-member' ); ?>"
+							aria-label="<?php esc_attr_e( stripslashes( $label ), 'ultimate-member' ); ?>"
 							<?php if ( $admin && count( $attrs['options'] ) > 1 ) { ?>multiple<?php } ?>
 						<?php echo $custom_dropdown; ?>>
 
@@ -685,7 +696,7 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 									<?php disabled( ! empty( $filter_from_url ) && in_array( $opt, $filter_from_url ) );
 
 									if ( $admin ) {
-										if ( is_string( $default_value ) ) {
+										if ( ! is_array( $default_value ) ) {
 											$default_value = array( $default_value );
 										}
 
@@ -710,7 +721,7 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 
 						<input type="hidden" id="<?php echo $filter; ?>_min" name="<?php echo $filter; ?>[]" class="um_range_min" value="<?php echo ! empty( $default_value ) ? esc_attr( min( $default_value ) ) : '' ?>" />
 						<input type="hidden" id="<?php echo $filter; ?>_max" name="<?php echo $filter; ?>[]" class="um_range_max" value="<?php echo ! empty( $default_value ) ? esc_attr( max( $default_value ) ) : '' ?>" />
-						<div class="um-slider" data-field_name="<?php echo $filter; ?>" data-min="<?php echo $range[0] ?>" data-max="<?php echo $range[1] ?>"></div>
+						<div class="um-slider" data-field_name="<?php echo $filter; ?>" data-min="<?php echo esc_attr( $range[0] ); ?>" data-max="<?php echo esc_attr( $range[1] ); ?>"></div>
 						<div class="um-slider-range" data-placeholder-s="<?php echo esc_attr( $single_placeholder ); ?>" data-placeholder-p="<?php echo esc_attr( $plural_placeholder ); ?>" data-label="<?php echo ( ! empty( $attrs['label'] ) ) ? esc_attr__( stripslashes( $attrs['label'] ), 'ultimate-member' ) : ''; ?>"></div>
 					<?php }
 
@@ -792,22 +803,23 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 		function slider_filters_range( $filter, $directory_data ) {
 			global $wpdb;
 
+			$range = false;
+
 			switch ( $filter ) {
 
 				default: {
 
 					$meta = $wpdb->get_row( $wpdb->prepare(
-						"SELECT MIN( meta_value ) as min_meta,
-						MAX( meta_value ) as max_meta,
+						"SELECT MIN( CONVERT( meta_value, DECIMAL ) ) as min_meta,
+						MAX( CONVERT( meta_value, DECIMAL ) ) as max_meta,
 						COUNT( DISTINCT meta_value ) as amount
 						FROM {$wpdb->usermeta}
 						WHERE meta_key = %s",
 						$filter
 					), ARRAY_A );
 
-					$range = false;
-					if ( isset( $meta['min_meta'] ) && isset( $meta['max_meta'] ) ) {
-						$range = array( $meta['min_meta'], $meta['max_meta'] );
+					if ( isset( $meta['min_meta'] ) && isset( $meta['max_meta'] ) && isset( $meta['amount'] ) && $meta['amount'] > 1 ) {
+						$range = array( (float) $meta['min_meta'], (float) $meta['max_meta'] );
 					}
 
 					$range = apply_filters( 'um_member_directory_filter_slider_common', $range, $directory_data, $filter );
@@ -839,13 +851,11 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 						MAX( meta_value ) as max_meta,
 						COUNT( DISTINCT meta_value ) as amount
 						FROM {$wpdb->usermeta}
-						WHERE meta_key = 'birth_date' AND 
-						      meta_value != ''",
+						WHERE meta_key = 'birth_date' AND
+							  meta_value != ''",
 					ARRAY_A );
 
-					if ( empty( $meta ) || ! isset( $meta['amount'] ) || $meta['amount'] === 1 ) {
-						$range = false;
-					} elseif ( isset( $meta['min_meta'] ) && isset( $meta['max_meta'] ) ) {
+					if ( isset( $meta['min_meta'] ) && isset( $meta['max_meta'] ) && isset( $meta['amount'] ) && $meta['amount'] > 1 ) {
 						$range = array( $this->borndate( strtotime( $meta['max_meta'] ) ), $this->borndate( strtotime( $meta['min_meta'] ) ) );
 					}
 
@@ -866,7 +876,8 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 		function slider_range_placeholder( $filter, $attrs ) {
 			switch ( $filter ) {
 				default: {
-					$label = ucwords( str_replace( array( 'um_', '_' ), array( '', ' ' ), $filter ) );
+					$label = ! empty( $attrs['label'] ) ? $attrs['label'] : $filter;
+					$label = ucwords( str_replace( array( 'um_', '_' ), array( '', ' ' ), $label ) );
 					$placeholders = apply_filters( 'um_member_directory_filter_slider_range_placeholder', false, $filter );
 
 					if ( ! $placeholders ) {
@@ -1223,7 +1234,7 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 			}
 
 			$this->query_args['number'] = ( ! empty( $directory_data['max_users'] ) && $directory_data['max_users'] <= $profiles_per_page ) ? $directory_data['max_users'] : $profiles_per_page;
-			$this->query_args['paged'] = ! empty( $_POST['page'] ) ? $_POST['page'] : 1;
+			$this->query_args['paged'] = ! empty( $_POST['page'] ) ? absint( $_POST['page'] ) : 1;
 		}
 
 
@@ -1235,7 +1246,7 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 		function sorting_query( $directory_data ) {
 			// sort members by
 			$this->query_args['order'] = 'ASC';
-			$sortby = ! empty( $_POST['sorting'] ) ? $_POST['sorting'] : $directory_data['sortby'];
+			$sortby = ! empty( $_POST['sorting'] ) ? sanitize_text_field( $_POST['sorting'] ) : $directory_data['sortby'];
 			$sortby = ( $sortby == 'other' ) ? $directory_data['sortby_custom'] : $sortby;
 
 			$custom_sort = array();
@@ -1249,24 +1260,10 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 				}
 			}
 
-			if ( $sortby == $directory_data['sortby_custom'] || in_array( $sortby, $custom_sort ) ) {
+			if ( 'username' == $sortby ) {
 
-				$custom_sort_type = apply_filters( 'um_member_directory_custom_sorting_type', 'CHAR', $sortby, $directory_data );
-
-				$this->query_args['meta_query'][] = array(
-					'relation' => 'OR',
-					$sortby . '_cs' => array(
-						'key'       => $sortby,
-						'compare'   => 'EXISTS',
-						'type'      => $custom_sort_type,
-					),
-					array(
-						'key'       => $sortby,
-						'compare'   => 'NOT EXISTS',
-					)
-				);
-
-				$this->query_args['orderby'] = array( $sortby . '_cs' => 'ASC', 'user_login' => 'ASC' );
+				$this->query_args['orderby'] = 'user_login';
+				$this->query_args['order'] = 'ASC';
 
 			} elseif ( 'display_name' == $sortby ) {
 
@@ -1333,6 +1330,25 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 				$this->query_args['orderby'] = array( 'last_name_c' => 'ASC', 'first_name_c' => 'ASC' );
 				unset( $this->query_args['order'] );
 
+			} elseif ( ( ! empty( $directory_data['sortby_custom'] ) && $sortby == $directory_data['sortby_custom'] ) || in_array( $sortby, $custom_sort ) ) {
+
+				$custom_sort_type = apply_filters( 'um_member_directory_custom_sorting_type', 'CHAR', $sortby, $directory_data );
+
+				$this->query_args['meta_query'][] = array(
+					'relation' => 'OR',
+					$sortby . '_cs' => array(
+						'key'       => $sortby,
+						'compare'   => 'EXISTS',
+						'type'      => $custom_sort_type,
+					),
+					array(
+						'key'       => $sortby,
+						'compare'   => 'NOT EXISTS',
+					)
+				);
+
+				$this->query_args['orderby'] = array( $sortby . '_cs' => 'ASC', 'user_login' => 'ASC' );
+
 			} else {
 
 				if ( strstr( $sortby, '_desc' ) ) {
@@ -1427,23 +1443,25 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 			if ( ! empty( $_POST['search'] ) ) {
 				// complex using with change_meta_sql function
 
+				$search = trim( stripslashes( sanitize_text_field( $_POST['search'] ) ) );
+
 				$meta_query = array(
 					'relation' => 'OR',
 					array(
-						'value'     => trim( stripslashes( $_POST['search'] ) ),
+						'value'     => $search,
 						'compare'   => '=',
 					),
 					array(
-						'value'     => trim( stripslashes( $_POST['search'] ) ),
+						'value'     => $search,
 						'compare'   => 'LIKE',
 					),
 					array(
-						'value'     => trim( serialize( strval( stripslashes( $_POST['search'] ) ) ) ),
+						'value'     => serialize( (string) $search ),
 						'compare'   => 'LIKE',
 					),
 				);
 
-				$meta_query = apply_filters( 'um_member_directory_general_search_meta_query', $meta_query, stripslashes( $_POST['search'] ) );
+				$meta_query = apply_filters( 'um_member_directory_general_search_meta_query', $meta_query, stripslashes( sanitize_text_field( $_POST['search'] ) ) );
 
 				$this->query_args['meta_query'][] = $meta_query;
 
@@ -1468,7 +1486,7 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 		function change_meta_sql( $sql, $queries, $type, $primary_table, $primary_id_column, $context ) {
 			if ( ! empty( $_POST['search'] ) ) {
 				global $wpdb;
-				$search = trim( stripslashes( $_POST['search'] ) );
+				$search = trim( stripslashes( sanitize_text_field( $_POST['search'] ) ) );
 				if ( ! empty( $search ) ) {
 
 					$meta_value = '%' . $wpdb->esc_like( $search ) . '%';
@@ -1531,6 +1549,7 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 			$filter_query = array();
 			if ( ! empty( $directory_data['search_fields'] ) ) {
 				$search_filters = maybe_unserialize( $directory_data['search_fields'] );
+
 				if ( ! empty( $search_filters ) && is_array( $search_filters ) ) {
 					$filter_query = array_intersect_key( $_POST, array_flip( $search_filters ) );
 				}
@@ -1545,6 +1564,12 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 
 			$this->is_search = true;
 			foreach ( $filter_query as $field => $value ) {
+				$field = sanitize_text_field( $field );
+				if ( is_array( $value ) ) {
+					$value = array_map( 'sanitize_text_field', $value );
+				} else {
+					$value = sanitize_text_field( $value );
+				}
 
 				$attrs = UM()->fields()->get_field( $field );
 				// skip private invisible fields
@@ -1593,13 +1618,12 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 								case 'text':
 
 									$value = stripslashes( $value );
-
 									$field_query = array(
 										'relation' => 'OR',
 										array(
 											'key'       => $field,
 											'value'     => trim( $value ),
-											'compare'   => 'LIKE',
+											'compare'   => apply_filters( 'um_members_directory_filter_text', 'LIKE', $field )
 										),
 									);
 
@@ -1612,22 +1636,22 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 										$field_query = array( 'relation' => 'OR' );
 
 										foreach ( $value as $single_val ) {
-											$single_val = stripslashes( $single_val );
+											$single_val = trim( stripslashes( $single_val ) );
 
 											$arr_meta_query = array(
 												array(
 													'key'       => $field,
-													'value'     => trim( $single_val ),
+													'value'     => $single_val,
 													'compare'   => '=',
 												),
 												array(
 													'key'       => $field,
-													'value'     => serialize( strval( trim( $single_val ) ) ),
+													'value'     => serialize( (string) $single_val ),
 													'compare'   => 'LIKE',
 												),
 												array(
 													'key'       => $field,
-													'value'     => '"' . trim( $single_val ) . '"',
+													'value'     => '"' . $single_val . '"',
 													'compare'   => 'LIKE',
 												)
 											);
@@ -1636,7 +1660,7 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 
 												$arr_meta_query[] = array(
 													'key'       => $field,
-													'value'     => serialize( intval( trim( $single_val ) ) ),
+													'value'     => serialize( (int) $single_val ),
 													'compare'   => 'LIKE',
 												);
 
@@ -1658,6 +1682,7 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 										'value'     => $value,
 										'compare'   => 'BETWEEN',
 										'inclusive' => true,
+										'type'		=> 'NUMERIC',
 									);
 
 									break;
@@ -1868,7 +1893,7 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 									$field_query = array(
 										'key'       => $field,
 										'value'     => $value,
-										'compare'   => '=',
+										'compare'   => apply_filters( 'um_members_directory_filter_text', '=', $field ),
 									);
 
 									break;
@@ -1881,20 +1906,22 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 									$field_query = array( 'relation' => 'OR' );
 
 									foreach ( $value as $single_val ) {
+										$single_val = trim( $single_val );
+
 										$arr_meta_query = array(
 											array(
 												'key'       => $field,
-												'value'     => trim( $single_val ),
+												'value'     => $single_val,
 												'compare'   => '=',
 											),
 											array(
 												'key'       => $field,
-												'value'     => serialize( strval( trim( $single_val ) ) ),
+												'value'     => serialize( (string) $single_val ),
 												'compare'   => 'LIKE',
 											),
 											array(
 												'key'       => $field,
-												'value'     => '"' . trim( $single_val ) . '"',
+												'value'     => '"' . $single_val . '"',
 												'compare'   => 'LIKE',
 											)
 										);
@@ -1903,7 +1930,7 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 
 											$arr_meta_query[] = array(
 												'key'       => $field,
-												'value'     => serialize( intval( trim( $single_val ) ) ),
+												'value'     => serialize( (int) $single_val ),
 												'compare'   => 'LIKE',
 											);
 
@@ -2062,7 +2089,7 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 		 */
 		function calculate_pagination( $directory_data, $total_users ) {
 
-			$current_page = ! empty( $_POST['page'] ) ? $_POST['page'] : 1;
+			$current_page = ! empty( $_POST['page'] ) ? absint( $_POST['page'] ) : 1;
 
 			$total_users = ( ! empty( $directory_data['max_users'] ) && $directory_data['max_users'] <= $total_users ) ? $directory_data['max_users'] : $total_users;
 
@@ -2207,7 +2234,7 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 			$dropdown_actions = $this->build_user_actions_list( $user_id );
 
 			$actions = array();
-			$can_edit = UM()->roles()->um_current_user_can( 'edit', $user_id ) || UM()->roles()->um_user_can( 'can_edit_everyone' );
+			$can_edit = UM()->roles()->um_current_user_can( 'edit', $user_id );
 
 			// Replace hook 'um_members_just_after_name'
 			ob_start();
@@ -2316,7 +2343,7 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 		function pagination_changes( $user_query ) {
 			global $wpdb;
 
-			$directory_id = $this->get_directory_by_hash( $_POST['directory_id'] );
+			$directory_id = $this->get_directory_by_hash( sanitize_key( $_POST['directory_id'] ) );
 			$directory_data = UM()->query()->post_data( $directory_id );
 
 			$qv = $user_query->query_vars;
@@ -2363,7 +2390,7 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 
 			global $wpdb;
 
-			$directory_id = $this->get_directory_by_hash( $_POST['directory_id'] );
+			$directory_id = $this->get_directory_by_hash( sanitize_key( $_POST['directory_id'] ) );
 			$directory_data = UM()->query()->post_data( $directory_id );
 
 			//predefined result for user without capabilities to see other members
@@ -2512,25 +2539,28 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 			 * @change_log
 			 * ["Since: 2.0"]
 			 * @usage
-			 * <?php add_filter( 'um_prepare_user_results_array', 'function_name', 10, 1 ); ?>
+			 * <?php add_filter( 'um_prepare_user_results_array', 'function_name', 10, 2 ); ?>
 			 * @example
 			 * <?php
-			 * add_filter( 'um_prepare_user_results_array', 'my_prepare_user_results', 10, 1 );
-			 * function my_prepare_user_results( $user_ids ) {
+			 * add_filter( 'um_prepare_user_results_array', 'my_prepare_user_results', 10, 2 );
+			 * function my_prepare_user_results( $user_ids, $query ) {
 			 *     // your code here
 			 *     return $user_ids;
 			 * }
 			 * ?>
 			 */
-			$user_ids = apply_filters( 'um_prepare_user_results_array', $user_ids );
+			$user_ids = apply_filters( 'um_prepare_user_results_array', $user_ids, $this->query_args );
 
 
 			$sizes = UM()->options()->get( 'cover_thumb_sizes' );
 
 			$this->cover_size = UM()->mobile()->isTablet() ? $sizes[1] : end( $sizes );
 
+			$this->cover_size = apply_filters( 'um_member_directory_cover_image_size', $this->cover_size, $directory_data );
+
 			$avatar_size = UM()->options()->get( 'profile_photosize' );
 			$this->avatar_size = str_replace( 'px', '', $avatar_size );
+			$this->avatar_size = apply_filters( 'um_member_directory_avatar_image_size', $this->avatar_size, $directory_data );
 
 			$users = array();
 			foreach ( $user_ids as $user_id ) {
@@ -2556,11 +2586,13 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 		 * @param string $element
 		 * @param string $trigger
 		 * @param array $items
+		 * @param string $parent
 		 */
-		function dropdown_menu( $element, $trigger, $items = array() ) {
+		function dropdown_menu( $element, $trigger, $items = array(), $parent = '' ) {
+			// !!!!Important: all links in the dropdown items must have "class" attribute
 			?>
 
-			<div class="um-new-dropdown" data-element="<?php echo $element; ?>" data-trigger="<?php echo $trigger; ?>">
+			<div class="um-new-dropdown" data-element="<?php echo $element; ?>" data-trigger="<?php echo $trigger; ?>" data-parent="<?php echo $parent; ?>">
 				<ul>
 					<?php foreach ( $items as $k => $v ) { ?>
 						<li><?php echo $v; ?></li>
@@ -2578,14 +2610,16 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 		 * @param string $element
 		 * @param string $trigger
 		 * @param string $item
+		 * @param string $additional_attributes
+		 * @param string $parent
 		 */
-		function dropdown_menu_js( $element, $trigger, $item ) {
+		function dropdown_menu_js( $element, $trigger, $item, $additional_attributes = '', $parent = '' ) {
 			?>
 
-			<div class="um-new-dropdown" data-element="<?php echo $element; ?>" data-trigger="<?php echo $trigger; ?>">
+			<div class="um-new-dropdown" data-element="<?php echo $element; ?>" data-trigger="<?php echo $trigger; ?>" data-parent="<?php echo $parent; ?>">
 				<ul>
 					<# _.each( <?php echo $item; ?>.dropdown_actions, function( action, key, list ) { #>
-						<li><a href="<# if ( typeof action.url != 'undefined' ) { #>{{{action.url}}}<# } else { #>javascript:void(0);<# }#>" class="{{{key}}}">{{{action.title}}}</a></li>
+						<li><a href="<# if ( typeof action.url != 'undefined' ) { #>{{{action.url}}}<# } else { #>javascript:void(0);<# }#>" class="{{{key}}}"<?php echo $additional_attributes ? " $additional_attributes" : '' ?>>{{{action.title}}}</a></li>
 					<# }); #>
 				</ul>
 			</div>
@@ -2594,11 +2628,15 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 		}
 
 
-
+		/**
+		 * AJAX handler - Get options for the member directory "Admin filtering"
+		 * @version 2.1.12
+		 */
 		function default_filter_settings() {
 			UM()->admin()->check_ajax_nonce();
 
-			$filter_key = sanitize_key( $_REQUEST['key'] );
+			// we can't use function "sanitize_key" because it changes uppercase to lowercase
+			$filter_key = sanitize_text_field( $_REQUEST['key'] );
 			$directory_id = absint( $_REQUEST['directory_id'] );
 
 			$html = $this->show_filter( $filter_key, array( 'form_id' => $directory_id ), false, true );
@@ -2606,4 +2644,4 @@ if ( ! class_exists( 'um\core\Member_Directory' ) ) {
 			wp_send_json_success( array( 'field_html' => $html ) );
 		}
 	}
-}
\ No newline at end of file
+}
